var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
import nodeFS from "fs";
import { glob } from "glob";
import nodePath from "path";
import { createLogger, normalizePath } from "vite";
import "crypto";
import "node:path";
import "node:fs";
const pkgName = "vite-plugin-react-app";
const pkgVersion = "0.1.2";
const officialLink = "https://github.com/FrontEndDev-org/vite-plugin-app";
function indentLine(line, size = 0) {
  return " ".repeat(size) + line;
}
function jsxLikeStringify(jsxLike) {
  const stringify = (jsx) => {
    const arg1 = jsx.props ? `{${Object.entries(jsx.props).map(([key, value]) => `${key}: ${value}`).join(", ")}}` : "null";
    const arg2 = jsx.children ? ", " + jsx.children.map(stringify).join(", ") : "";
    return `createElement(${jsx.tag}, ${arg1}${arg2})`;
  };
  return stringify(jsxLike);
}
class PageRoute {
  constructor(fsNode) {
    __publicField(this, "parent");
    this.fsNode = fsNode;
  }
  get id() {
    return this.fsNode.id;
  }
  get path() {
    const paths = [this.fsNode.path];
    const find = (p) => {
      if (!p) return;
      if (p.group && p.depth === 1) {
        paths.unshift(p.group);
        return;
      }
      if (!p.group && p.layout) return;
      paths.unshift(p.path);
      find(p.parent);
    };
    find(this.fsNode.parent);
    return paths.length === 0 || paths.length === 1 && paths.at(0) === "" ? "" : nodePath.join(...paths);
  }
  render(indent = 0, tabSize = 4) {
    const indent_1 = indent * tabSize;
    const indent_2 = (indent + 1) * tabSize;
    const lines = [
      //
      indentLine("{", indent_1)
    ];
    lines.push(indentLine(`path: ${JSON.stringify(this.path)},`, indent_2));
    const pageName = FSRouter.makePageComponentName(this.fsNode.id);
    const element = jsxLikeStringify({
      tag: "Suspense",
      props: {},
      children: [{ tag: pageName }]
    });
    lines.push(indentLine(`element: ${element},`, indent_2));
    lines.push(indentLine("},", indent_1));
    return lines;
  }
}
class LayoutRoute {
  constructor(fsNode) {
    __publicField(this, "parent");
    __publicField(this, "children", []);
    this.fsNode = fsNode;
  }
  get id() {
    return this.fsNode.id;
  }
  render(indent = 0, tabSize = 4) {
    const indent_1 = indent * tabSize;
    const indent_2 = (indent + 1) * tabSize;
    const lines = [
      //
      indentLine("{", indent_1)
    ];
    const layoutName = FSRouter.makeLayoutComponentName(this.fsNode.id);
    const element = jsxLikeStringify({
      tag: "Suspense",
      props: {},
      children: [{ tag: layoutName }]
    });
    lines.push(indentLine(`element: ${element},`, indent_2));
    if (this.children.length) {
      lines.push(indentLine("children: [", indent_2));
      lines.push(...this.children.flatMap((child) => child.render(indent + 2, tabSize)));
      lines.push(indentLine("],", indent_2));
    }
    lines.push(indentLine("},", indent_1));
    return lines;
  }
}
class FSRouter {
  constructor(fsTree) {
    __publicField(this, "pageRoutes", /* @__PURE__ */ new Map());
    __publicField(this, "layoutRoutes", /* @__PURE__ */ new Map());
    this.fsTree = fsTree;
  }
  addPageNode(fsNode) {
    this.pageRoutes.set(fsNode.id, new PageRoute(fsNode));
  }
  addLayoutNode(fsNode) {
    this.layoutRoutes.set(fsNode.id, new LayoutRoute(fsNode));
  }
  build() {
    this.pageRoutes.forEach((pageRoute) => {
      const find = (node) => {
        if (node.layout) {
          const layoutRoute = this.layoutRoutes.get(node.id);
          if (layoutRoute) {
            pageRoute.parent = layoutRoute;
            layoutRoute.children.push(pageRoute);
            return;
          }
        }
        if (node.parent) find(node.parent);
      };
      find(pageRoute.fsNode);
    });
    this.layoutRoutes.forEach((layoutRoute) => {
      const find = (node) => {
        const parent = node.parent;
        if (!parent) return;
        if (node.group && node.depth === 1) return;
        if (parent.layout) {
          const parentRoute = this.layoutRoutes.get(parent.id);
          if (parentRoute) {
            layoutRoute.parent = parentRoute;
            parentRoute.children.push(layoutRoute);
            return;
          }
        }
        find(parent);
      };
      find(layoutRoute.fsNode);
    });
  }
  render(tabSize = 4) {
    const importPages = ["// pages"];
    const importLayouts = ["// layouts"];
    const pageLines = [];
    this.pageRoutes.forEach((route) => {
      const importName = FSRouter.makePageComponentName(route.id);
      importPages.push(`const ${importName} = lazy(() => import("${route.fsNode.page}"));`);
      if (!route.parent) {
        pageLines.push(...route.render(0, tabSize));
      }
    });
    const layoutLines = [];
    this.layoutRoutes.forEach((route) => {
      const importName = FSRouter.makeLayoutComponentName(route.id);
      importLayouts.push(`const ${importName} = lazy(() => import("${route.fsNode.layout}"));`);
      if (!route.parent) {
        layoutLines.push(...route.render(0, tabSize));
      }
    });
    return [
      "/**",
      " * generated by pkg-name-for-test@pkg-version-for-test",
      " * @ref https://github.com/FrontEndDev-org/vite-plugin-app",
      " * @ref https://reactrouter.com/",
      " */",
      "",
      'import { Suspense, lazy, createElement } from "react";',
      'import { Outlet } from "react-router-dom";',
      "",
      ...importPages,
      "",
      ...importLayouts,
      "",
      "export const routes = [",
      ...pageLines,
      ...layoutLines,
      "];",
      ""
    ].join("\n");
  }
  // TODO
  update() {
  }
  static makeLayoutComponentName(id) {
    return "Layout_" + id;
  }
  static makePageComponentName(id) {
    return "Page_" + id;
  }
}
class FSNode {
  constructor(tree, pageMeta) {
    __publicField(this, "id");
    /**
     * 分组
     */
    __publicField(this, "group");
    /**
     * 深度，起始为 0
     */
    __publicField(this, "depth");
    /**
     * 【唯一】所在目录，相对于 cwd
     */
    __publicField(this, "dir");
    /**
     * 路径名
     */
    __publicField(this, "name");
    /**
     * 页面，未声明页面时为空字符串
     * @ref https://reactrouter.com/en/main/route/route#elementcomponent
     */
    __publicField(this, "page");
    /**
     * https://reactrouter.com/en/main/start/concepts#layout-routes
     */
    __publicField(this, "layout");
    /**
     * @ref https://reactrouter.com/en/main/route/loader
     */
    __publicField(this, "loader");
    /**
     * @ref https://reactrouter.com/en/main/route/action
     */
    __publicField(this, "action");
    /**
     * @ref https://reactrouter.com/en/main/route/error-element
     */
    __publicField(this, "error");
    /**
     * @ref https://react.dev/reference/react/Suspense
     */
    __publicField(this, "loading");
    __publicField(this, "children", []);
    __publicField(this, "parent");
    this.tree = tree;
    const { page, depth, group, dir, name } = pageMeta;
    this.id = this.tree.nextId();
    this.group = group;
    this.depth = depth;
    this.dir = dir;
    this.name = name;
    this.page = page ? nodePath.join(tree.cwd, page) : "";
    this.layout = this._resolveFile(["layout.tsx", "layout.jsx"]);
    this.loader = this._resolveFile(["loader.tsx", "loader.jsx"]);
    this.action = this._resolveFile(["action.tsx", "action.jsx"]);
    this.error = this._resolveFile(["error.tsx", "error.jsx"]);
    const depthRoutes = this.tree.defaultMap.get(depth) || [];
    depthRoutes.push(this);
    this.tree.defaultMap.set(depth, depthRoutes);
    this.tree.dirMap.set(dir, this);
    if (group) {
      const groupRoutes = this.tree.groupMap.get(group) || [];
      groupRoutes.push(this);
      this.tree.groupMap.set(group, groupRoutes);
    }
    if (this.page) {
      this.tree.fsRouter.addPageNode(this);
    }
    if (this.layout) {
      this.tree.fsRouter.addLayoutNode(this);
    }
    if (depth === 0) {
      this.tree.root = this;
    }
    this._linking();
  }
  _resolveFile(fileNames) {
    const index = this.tree.options.resolveExist(this, fileNames);
    const fileName = fileNames[index];
    return fileName && nodePath.join(this.tree.cwd, this.dir, fileName);
  }
  update(meta) {
    Object.assign(this, meta);
  }
  get path() {
    return this.name === "." ? "" : this.name.startsWith("[") && this.name.endsWith("]") ? ":" + this.name.slice(1, -1) : this.name;
  }
  _linking() {
    const { depth, dir } = this;
    const parentDepth = depth - 1;
    const parentDir = nodePath.dirname(dir);
    const parentName = nodePath.basename(parentDir);
    const parentGroup = depth > 0 ? this.group : void 0;
    if (depth < 1) return;
    const parent = this.tree.dirMap.get(parentDir);
    if (parent) {
      this.parent = parent;
      parent.children.push(this);
    } else {
      const newParent = new FSNode(this.tree, {
        depth: parentDepth,
        page: "",
        dir: parentDir,
        name: parentName,
        group: parentGroup
      });
      this.parent = newParent;
      newParent.children.push(this);
    }
  }
  static parse(page) {
    const segments = page.split("/");
    segments.pop();
    const depth = segments.length;
    const name = segments.at(-1) || ".";
    const dir = segments.join("/") || ".";
    const firstSeg = segments.at(0);
    const group = depth > 0 && (firstSeg == null ? void 0 : firstSeg.startsWith("(")) && (firstSeg == null ? void 0 : firstSeg.endsWith(")")) ? firstSeg == null ? void 0 : firstSeg.slice(1, -1) : void 0;
    return { page, depth, dir, name, group };
  }
}
class FSTree {
  constructor(options) {
    __publicField(this, "root");
    __publicField(this, "dirMap", /* @__PURE__ */ new Map());
    __publicField(this, "defaultMap", /* @__PURE__ */ new Map());
    __publicField(this, "groupMap", /* @__PURE__ */ new Map());
    __publicField(this, "idSet", /* @__PURE__ */ new Set());
    __publicField(this, "fsRouter", new FSRouter(this));
    __publicField(this, "_id", 0);
    this.options = options;
  }
  get cwd() {
    return this.options.cwd;
  }
  mount(pages) {
    pages.forEach((page) => {
      const meta = FSNode.parse(page);
      const fsPage = this.dirMap.get(meta.dir);
      if (fsPage) {
        fsPage.update(meta);
      } else {
        new FSNode(this, meta);
      }
    });
    this.fsRouter.build();
  }
  // TODO
  update(pages, parent) {
  }
  render(tabSize = 4) {
    return this.fsRouter.render(tabSize);
  }
  nextId() {
    return this._id++;
  }
  static makeLayoutComponentName(id) {
    return "Layout_" + id;
  }
  static makePageComponentName(id) {
    return "Page_" + id;
  }
}
class ReactApp {
  constructor(options, cwd = process.cwd()) {
    __publicField(this, "logger");
    __publicField(this, "fsTree");
    __publicField(this, "absApp");
    this.options = options;
    this.cwd = cwd;
    this.logger = createLogger(options.logLevel, { prefix: `[${pkgName}]` });
    this.logger.info(`new Options: ${JSON.stringify(options)}`);
    this.absApp = nodePath.join(this.cwd, options.appDir);
    this.fsTree = new FSTree({
      cwd: nodePath.join("/", options.appDir),
      resolveExist: (page, fileNames) => {
        return fileNames.findIndex((fileName) => {
          const absFile = nodePath.join(this.absApp, page.dir, fileName);
          return nodeFS.existsSync(absFile) && nodeFS.statSync(absFile).isFile();
        });
      }
    });
  }
  get output() {
    const output = this.fsTree.render();
    this.logger.info(`Generated: ${output}`);
    return output;
  }
  _inApp(path) {
    return path.startsWith(this.absApp);
  }
  setup({ watcher }) {
    this.logger.info(`setup`);
    watcher.on("unlink", async (path) => {
      if (!this._inApp(path)) return;
      await this.removeFile(path);
    });
    watcher.on("add", async (path) => {
      if (!this._inApp(path)) return;
      await this.addFile(path);
    });
    watcher.on("change", async (path) => {
      if (!this._inApp(path)) return;
      await this.updateFile(path);
    });
    this.start();
  }
  start() {
    this.logger.info(`start`);
    const pageFiles = glob.sync("**/page.{jsx,tsx}", {
      cwd: this.absApp,
      nodir: true,
      dot: false
    }).map(normalizePath);
    this.logger.info(`Page files: ${JSON.stringify(pageFiles)}`);
    this.fsTree.mount(pageFiles);
  }
  async addFile(file) {
    this.logger.info(`Add file: ${file}`);
    this.start();
  }
  async updateFile(file) {
    this.logger.info(`Update file: ${file}`);
    this.start();
  }
  async removeFile(file) {
    this.logger.info(`Remove file: ${file}`);
    this.start();
  }
}
const virtualModuleId = "virtual:react-app";
const resolvedVirtualModuleId = "\0" + virtualModuleId;
function reactApp(userPluginOptions = {}) {
  let reactApp2 = null;
  return {
    name: pkgName,
    enforce: "pre",
    async configResolved(config) {
      const defaults = {
        debug: false,
        appDir: "src/app",
        logLevel: "error",
        disableLazy: false,
        caseSensitive: false
      };
      reactApp2 = new ReactApp(Object.assign(defaults, userPluginOptions), config.root);
    },
    configureServer(server) {
      reactApp2 == null ? void 0 : reactApp2.setup(server);
    },
    resolveId(id) {
      if (id !== virtualModuleId) return;
      return resolvedVirtualModuleId;
    },
    async load(id) {
      if (id !== resolvedVirtualModuleId) return;
      return reactApp2 == null ? void 0 : reactApp2.output;
    }
  };
}
export {
  officialLink,
  pkgName,
  pkgVersion,
  reactApp
};
//# sourceMappingURL=index.mjs.map
